Package: breakpointR
Type: Package
Title: Find breakpoints in Strand-seq data
Version: 1.29.0
Date: 2021-11-20
Author: David Porubsky, Ashley Sanders, Aaron Taudt
Maintainer: David Porubsky <david.porubsky@gmail.com>
Description: This package implements functions for finding breakpoints, plotting and export of Strand-seq data.
Depends: R (>= 3.5), GenomicRanges, cowplot, breakpointRdata
Imports: methods, utils, grDevices, stats, S4Vectors, GenomeInfoDb (>=
        1.12.3), IRanges, Rsamtools, GenomicAlignments, ggplot2,
        BiocGenerics, gtools, doParallel, foreach
Suggests: knitr, BiocStyle, testthat
License: file LICENSE
LazyLoad: yes
VignetteBuilder: knitr
Packaged: 2025-10-31 20:50:13 UTC; biocbuild
biocViews: Software, Sequencing, DNASeq, SingleCell, Coverage
URL: https://github.com/daewoooo/BreakPointR
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/breakpointR
git_branch: devel
git_last_commit: 5469268
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
