# =======================================================
# Some global variables that can be used in all functions
# =======================================================

#' @import IRanges
#' @import GenomeInfoDb
#' @import GenomicRanges
#' @import breakpointRdata
#' @importFrom methods as is
NULL

class.breakpoint <- "BreakPoint"
